<?php
namespace Zeedhi\Notification;

/**
 * class Message
 *
 * Message model
 *
 * @package Zeedhi\Notification
 */
class Message {

    /** @var string Message identifier */
    private $id;
    /** @var string Topic identifier that the message belongs to */
    private $topicId;
    /** @var array Notification data  */
    private $notification;
    /** @var int Number of users that have received the message */
    private $nrReceived;
    /** @var int Number of users that have read the message */
    private $nrRead;

    /**
     * Message constructor.
     *
     * @param string $id
     * @param string $topicId
     * @param string $notification
     * @param int    $nrRead
     * @param int    $nrReceived
     */
    public function __construct($id, $topicId, $notification, $nrReceived, $nrRead) {
        $this->id = $id;
        $this->topicId = $topicId;
        $this->notification = $notification;
        $this->nrRead = $nrRead;
        $this->nrReceived = $nrReceived;
    }

    /**
     * getId
     *
     * Get identifier value
     *
     * @return string Identifier value
     */
    public function getId() {
        return $this->id;
    }

    /**
     * getNrReceived
     *
     * Get number of users that have received the message
     *
     * @return int Number of users
     */
    public function getNrReceived() {
        return $this->nrReceived;
    }

    /**
     * getNrRead
     *
     * Get number of users that have read the message
     *
     * @return int Number of users
     */
    public function getNrRead() {
        return $this->nrRead;
    }

    /**
     * getNotification
     *
     * Get notification data
     *
     * @return array Notification data
     */
    public function getNotification() {
        return $this->notification;
    }

    /**
     * getTopicId
     *
     * Get topic identifier that the message belongs to
     *
     * @return string Topic identifier
     */
    public function getTopicId() {
        return $this->topicId;
    }
}