import { Metadata, Http, IEventParam, IEvent } from '@zeedhi/core';
import { Modal, ModalService, Form, Grid, DialogService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

Metadata.get('metadata/ColorPalette/colorPalette.json');
Metadata.get('metadata/ColorPalette/Modal/color.json');
Metadata.get('metadata/ColorPalette/Modal/colorChange.json');

export class ColorPaletteController {

  private colorPaletteModal!: Modal;
  private colorPaletteChangeModal!: Modal;
  public hexadecimalColor: string = '#000000';
  public order = 0;
  public requestColor: any = '';
  public rgb: any = '';
  public hsv: any = '';
  public id = '';
  public row: any = '';
  public cont = 0;

  public async addClick() {
    this.order += this.cont;
    await this.createColorPaletteModal();
    await this.openColorPaletteModal();
    this.findColor();
  }

  public async createColorPaletteModal() {
    if (!this.colorPaletteModal) {
      this.colorPaletteModal = await ModalService.createFromJson('ColorPalette/Modal/color', true);
    }
  }

  public async createColorPaletteChangeModal() {
    if (!this.colorPaletteChangeModal) {
      this.colorPaletteChangeModal = await ModalService.createFromJson('ColorPalette/Modal/colorChange', true);
    }
  }

  public openColorPaletteModal() {
    this.colorPaletteModal.show();
  }

  public closeColorPaletteModal() {
    this.colorPaletteModal.hide();
  }

  public openColorPaletteChangeModal() {
    this.colorPaletteChangeModal.show();
  }

  public closeColorChangePaletteModal() {
    this.colorPaletteChangeModal.hide();
  }

  public setOrder() {
    const form = Metadata.getInstance<Form>('paletteColorForm');
    form.value.ordem = this.order;
  }

  public setOrderChangeColor() {
    const form = Metadata.getInstance<Form>('paletteColorChangeForm');
    form.value.ordem = this.order;
  }

  public hexToRgb(hexadecimalColor: any) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hexadecimalColor);
    return result ? {
      r: parseInt(result[1], 16),
      g: parseInt(result[2], 16),
      b: parseInt(result[3], 16)
    } : null;
  }

  public rgbToHsv(r: any, g: any, b: any) {
    const result = '';
    r /= 255;
    g /= 255;
    b /= 255;
    const max = Math.max(r, g, b);
    const min = Math.min(r, g, b);
    let h = max;
    let s = max;
    let v = max;
    const d = max - min;
    s = max === 0 ? 0 : (d / max);
    if (max === min) {
      h = 0;
    } else {
      switch (max) {
        case r:
          h = (g - b) / d + (g < b ? 6 : 0);
          break;
        case g:
          h = (b - r) / d + 2;
          break;
        case b:
          h = (r - g) / d + 4;
          break;
        default:
      }
    }
    h /= 6;
    v *= 100;
    return [h, s, v.toString()];
  }

  public getHexadecimalColor({row}: any) {
    return { backgroundColor: row.color, width: '100%' };
  }

  public selectedCheckboxRows({ component, event }: any) {
    event?.preventDefault();
    this.row = component.selectedRows;
    this.showMessageDeleteColor();
  }

  private back: any = {
    click: () => DialogService.hide(),
  };

  public showMessageDeleteColor() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação da operação',
      text: 'Deseja excluir esta cor?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteColor(); DialogService.hide(); } }, outline: true },
        { name: 'no', component: 'ZdButton', label: 'Não', events: this.back as any },
      ],
    });
  }

  public async findColor() {
    this.setOrder();
    const uriFindColor = '/findColor';
    this.requestColor = await Http.get(uriFindColor);
  }

  public async findColorChangeColor() {
    this.setOrderChangeColor();
    const uriFindColor = '/findColor';
    this.requestColor = await Http.get(uriFindColor);
  }

  public async saveColor() {
    this.closeColorPaletteModal();
    const paletteColorGrid = Metadata.getInstance<TekGrid>('paletteColorTekGrid');
    const form = Metadata.getInstance<Form>('paletteColorForm');
    const uriSaveColor = '/saveColor';
    this.rgb = this.hexToRgb(this.hexadecimalColor);
    this.hsv = this.rgbToHsv(this.rgb.r, this.rgb.g, this.rgb.b);
    const requestType = {requestType: 'Row', row: {_id: null, order: form.value.ordem, changed: true, color: {_originalInput: {h: this.hsv[0], s: this.hsv[1], v: this.hsv[2].concat('%'), __index: -1}, _r: this.rgb.r, _a: 1, _b: this.rgb.b, _format: 'hsv', _g: this.rgb.g, _ok: true, _roundA: 1}, colorHex: this.hexadecimalColor}, origin: {containerLabel: "Color Palette", widgetLabel: "Color"}};
    await Http.post(uriSaveColor, requestType);
    paletteColorGrid.reload();
    this.cont += 10;
  }

  public async deleteColor() {
    const paletteColorGrid = Metadata.getInstance<TekGrid>('paletteColorTekGrid');
    const uriDeleteColor = '/deleteColor';
    for (let i = 0; i < this.row.length; i += 1) {
       let requestType = {};
       requestType = {requestType: 'Row', origin: {containerLabel: "Color Palette", widgetLabel: "Color"}, row: {color: this.row[i].color, order: this.row[i].order, __isSelected: false, __notSelectable: false, _id: this.row[i]._id}};
       Http.post(uriDeleteColor, requestType);
       paletteColorGrid.reload();
    }
    await paletteColorGrid.reload();
  }

  public async changeColor(params: any) {
    await this.createColorPaletteChangeModal();
    await this.openColorPaletteChangeModal();
    this.findColorChangeColor();
    const paletteColorGrid = Metadata.getInstance<TekGrid>('paletteColorTekGrid');
    const form = Metadata.getInstance<Form>('paletteColorChangeForm');
    form.value.corHexadecimal = params.row.color;
    form.value.ordem = params.row.order;
    this.id = params.row._id;
  }

  public async editColor() {
    this.closeColorChangePaletteModal();
    const paletteColorGrid = Metadata.getInstance<TekGrid>('paletteColorTekGrid');
    const uriSaveColor = '/saveColor';
    const form = Metadata.getInstance<Form>('paletteColorChangeForm');
    this.rgb = this.hexToRgb(form.value.cor);
    this.hsv = this.rgbToHsv(this.rgb.r, this.rgb.g, this.rgb.b);
    const requestType = {requestType: 'Row', row: {_id: this.id, order: form.value.ordem, changed: true, color: {_originalInput: {h: this.hsv[0], s: this.hsv[1], v: this.hsv[2].concat('%'), __index: -1}, _r: this.rgb.r, _a: 1, _b: this.rgb.b, _format: 'hsv', _g: this.rgb.g, _ok: true, _roundA: 1}, colorHex: form.value.cor}, origin: {containerLabel: "Color Palette", widgetLabel: "Color"}};
    await Http.post(uriSaveColor, requestType);
    paletteColorGrid.reload();
  }
}
